/*
    AllegroBobble - An implementation of the classic game using the Allegro
    library (version 4) and the Allegro Wrapper
    Copyright (C) 2013, Alessandro Francesco De Rosis

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "PuzzleBubble.h"
#include <sstream>



PuzzleBubble::PuzzleBubble(): Allegro(640, 480, 24, false, false),
                              freccia(),
                              backgroundLayer(640,480),
                              middleLayer(640,480),
                              topLayer(640,480),
                              gameOver(false),
                              level(1),
                              levelText("Level: 1", Point(520,180), Color(255,255,255)),               
                              point(0),
                              pointText("Point: 0", Point(520,210), Color(255,255,255)),
                              game_over("gameover2.wav", 200),
                              backgroundSound("backgroundSound.wav",255),
                              shot("laser_gun.wav",170),
                              y_muro(10)
{
   add(DrawablePointer(backgroundLayer));
   add(DrawablePointer(middleLayer));
   add(DrawablePointer(topLayer));
   for(int i=30;i<=420;i+=30)
      backgroundLayer.add(Sprite("wall2.bmp", Point(i, 430)));
   for(int i = 30; i < 480; i += 30) 
   {
      backgroundLayer.add(Sprite("wall.bmp", Point(10, i)));
      backgroundLayer.add(Sprite("wall.bmp", Point(450, i)));
      backgroundLayer.add(Sprite("wall.bmp", Point(450, i)));
   }
   aggiungiRiga();
   middleLayer.setNotification(ON_MOUSE_MOVE).setNotification(ON_TICK);
   middleLayer.add(DrawablePointer(freccia));
   backgroundLayer.add(Sprite("v.bmp",Point(31,68)));
   aggiungiADaInserire();
   backgroundLayer.add(Text("Press Q to exit", Point(31, 470), Color(255,255,255)));

   inserite.push_back(new Ball(true,*this,Point(60,-20)));
   middleLayer.add(DrawablePointer(*inserite.back()));
   inserite.push_back(new Ball(true,*this,Point(120,-20)));   
   middleLayer.add(DrawablePointer(*inserite.back()));
   inserite.push_back(new Ball(true,*this,Point(180,-20)));
   middleLayer.add(DrawablePointer(*inserite.back()));
   inserite.push_back(new Ball(true,*this,Point(240,-20)));
   middleLayer.add(DrawablePointer(*inserite.back()));
   inserite.push_back(new Ball(true,*this,Point(300,-20)));
   middleLayer.add(DrawablePointer(*inserite.back()));
   inserite.push_back(new Ball(true,*this,Point(360,-20)));
   middleLayer.add(DrawablePointer(*inserite.back()));




   inserite.push_front(new Ball(*this,Point(30,30), "sfera1.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(90,30),"sfera2.bmp"));   
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(150,30),"sfera3.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(210,30),"sfera1.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(270,30),"sfera2.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(330,30),"sfera3.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
   inserite.push_front(new Ball(*this,Point(390,30),"sfera1.bmp"));
   middleLayer.add(DrawablePointer(*inserite.front()));
    
   middleLayer.add(DrawablePointer(*daInserire.front()));
                           
   backgroundLayer.add(DrawablePointer(levelText));
   backgroundLayer.add(DrawablePointer(pointText));
   backgroundSound.setLoop(true);
   backgroundSound.play();
}

void PuzzleBubble:: aggiungiADaInserire()
{
   for(int i=0;i<20;i++)
     daInserire.push(new Ball(*this,Point(-200,-200)));
}

void PuzzleBubble:: scala()
{
    for(list<Ball*>::iterator i=inserite.begin();i!=inserite.end();i++)
       (*i)->setLeftTopCorner(Point((*i)->getLeftTopCorner().getX(),(*i)->getLeftTopCorner().getY()+20));
}
void PuzzleBubble:: aggiungiRiga()
{  
   if(y_muro>10)
     scala();
   for(int i=0;i<450;i+=115)
   {
     middleLayer.add(Sprite("wall1.bmp", Point(10+i, y_muro)));
   }
   y_muro+=20;
}

PuzzleBubble::~PuzzleBubble()
{
   while(daInserire.empty())
    {
        delete daInserire.front();
        daInserire.pop();
    }
    for(list<Ball*>:: iterator i=inserite.begin();i!=inserite.end();i++)
        delete *i;
  
}
void PuzzleBubble::onTick()
{
   

    daInserire.front()->setLeftTopCorner(Point(freccia.get2X()-30,freccia.get2Y()-30)); 
    
    Allegro::onTick();
    if(getKeyboard().isKeyPressed(KEY_Q))
        exit = true;
    if(gameOver)
      return;

      if(getMouse().isLeftButtonPressed())
      {
          shot.play();
          static int numeroDiLanci=1;
          daInserire.front()->lancia();
          if(numeroDiLanci%8==0)
             aggiungiRiga();
          if(numeroDiLanci%4==0 && level<5)
          {
             stringstream ss;
             ss << "Level: " << ++level;
             levelText.setText(ss.str());
          }
       
          numeroDiLanci++;
       } 
      
    if(isGameOver())
    {
        draw();
        backgroundSound.stop();
        middleLayer.unsetNotification(ALL);
        game_over.play();        
        topLayer.add(Text("GAME OVER", Point(640/2-30, 480/2), Color(255,0,0), Text::ALIGN_CENTER, Font("arcanum_bold_48.tga")));
        

    }
    draw();
}

bool PuzzleBubble::isGameOver() 
{
    for(list<Ball*>::iterator i=inserite.begin();i!=inserite.end();i++)
         if((*i)->getLeftTopCorner().getY()+60>430 )
         {
            gameOver=true;
            return true;
         }
    return false;
}

